function varargout = get_goes(varargin)
% Gets Geostationary Operational Environmental Satellite (GOES) 5-km data.
% from NOAA archive.
%
% get_goes
% get_goes(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007
% $Version 2.1.2$

%==========================================================================
% Meri Sheremet
% May 2007
%
% REVISION HISTORY:
% 2007/01/01 created, ms
%==========================================================================

if nargin<1
% no arguments passed, get parameters from GUI
h = gui_goes;
if nargout
    varargout{1} = h; %set figure handle as output
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument request, get GOES data  

URLBase='http://oceanwatch.pfeg.noaa.gov:8081/thredds/dodsC/satellite/GA/ssta';
MBLIMIT=100;

request=varargin{1};
request.RequestDate=datestr(now,'yyyy-mm-dd HH:MM:SS');
R=request;

%assignin('base','request',request);

if(exist('loaddap')==0)
  errordlg('loaddap not found. Make sure it is in your MATLAB path.')
return
end
     
% get available lat,lon,time 
%loaddap('http://oceanwatch.pfeg.noaa.gov:8081/thredds/dodsC/satellite/GA/ssta/1day?time')
URL=[URLBase '/' R.TAVG '?lat,lon,time'];
loaddap(['+v','-e'],URL);
%dods_err = 0 means no error.
%dods_err = 1 means error.
%dods_err_msg [variable where error message is stored]
if dods_err
    disp(dods_err_msg)
    msgbox('Could not access dataset site. Sorry, try again later.');
    close(hwaitbar);
    return
end
% make sure that -180<lon<180
%if max(lon) > 180 lon=lon-360; end 

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

% adjust entered longitudes modulo 360 to match the dataset range 
LonM=0.5*(lon(1)+lon(end));
R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;

iLAT1=min(find(lat >= R.LAT1))-1;
iLAT2=max(find(lat <= R.LAT2))-1;
iLON1=min(find(lon >= R.LON1))-1;
iLON2=max(find(lon <= R.LON2))-1;

if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end

if(isempty(R.LATINCR)|isempty(R.LONINCR)|isempty(R.DATEINCR)|R.LATINCR==0|R.LONINCR==0|R.DATEINCR==0) 
    errordlg(['Subsampling values are invalid.']);
    return
end
if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end

if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['Select how you would like to save data.']);
    return
end

if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end

if R.Fields{1,2}=='n' 
    errordlg(['No variables have been selected.']);
    return
end
 
%CALT= '[0:1:0]';   % ConstraintAltitude
CLAT=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
CLON=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude
CALT= '[0:1:0]';   % ConstraintAltitude

% According to the attributes  
% A=loaddap('-A','http://oceanwatch.pfeg.noaa.gov:8081/thredds/dodsC/satellite/GA/ssta/1day?time')
% the units of time are
% A.time.units: '"seconds since 1970-01-01T00:00:00Z"'
% Convert time to days since 0000 01 01
% The time corresponds to the end of the averaging interval
% 
timeJD=datenum(1970,01,01)+time/86400;

% dt 
%dt=1;
%if strcmp(TAVG,'1day')         dt=1;
%elseif strcmp(TAVG,'3day')     dt=1;
%elseif strcmp(TAVG,'8day')     dt=1;
%elseif strcmp(TAVG,'14day')    dt=1;
%elseif strcmp(TAVG,'mday')     dt=30;    
%end

JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd');
% and find indices that in the timeJD that fall between JD1 and JD2+1 (day)
iJD1=min(find(timeJD >= JD1))-1;
iJD2=max(find(timeJD <= JD2+1))-1;
%iJD1=max(find(timeJD-dt/2 <= JD1))-1;
%iJD2=min(find(timeJD+dt/2 >= JD2))-1;

if(isempty(iJD1) | isempty(iJD2)) | (iJD2<iJD1)
    errordlg(['Requested time limits contain no data.']);
    return
end

CDATE=['[' num2str(iJD1) ':' num2str(R.DATEINCR) ':' num2str(iJD2) ']']; % ConstraintDate

% Estimate the amount of requested data
MB=(iJD2-iJD1+1)/R.DATEINCR*(iLAT2-iLAT1+1)/R.LATINCR*(iLON2-iLON1+1)/R.LONINCR*8/1024/1024;
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'GOES Requested Data Amount','Continue','Cancel','Cancel');
if strcmp(ButtonName,'Cancel') return; end
end

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end

hwaitbar=waitbar(0,'Acquiring ','Name','GOES Download Progress');
waitbar( 0.0 , hwaitbar, ['Acquiring GAssta'],'Name','GOES Download Progress');

URL=[URLBase '/' R.TAVG '?GAssta'];
disp(URL)
A=loaddap('-A',URL); % Get Attributes

waitbar( 0.5 , hwaitbar, ['Acquiring GAssta'],'Name','GOES Download Progress');

URL=[URLBase '/' R.TAVG '?GAssta' CDATE CALT CLAT CLON];
%disp(URL)
loaddap(['+v','-e'],URL);
%dods_err = 0 means no error.
%dods_err = 1 means error.
%dods_err_msg [variable where error message is stored]
if dods_err
    disp(dods_err_msg)
    msgbox('Could not access dataset site. Sorry, try again later.');
    close(hwaitbar);
    return
end


% replace fill values (-1.e32, -9999999) with NaN
j=find(GAssta.GAssta < -9999998 );
GAssta.GAssta(j)=NaN;

% add Gregorian time to the GAssta structure
t=GAssta.time;
tJD=datenum(1970,01,01,0,0,0)+t/86400;
GAssta.user_friendly_time=datestr(tJD,'yyyy-mm-dd HH:MM:SS');
% add url field to the GAssta structure
GAssta.url=URL;
% include attributes into the GAssta structure
GAssta.attributes=A;

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'native')==1 
    assignin('base','GAssta',GAssta)  ;  
end
if R.SaveFiles=='y' & strcmp(R.SaveMode,'native')==1 
    fn0=[R.DIRNAME '/' R.FNPREFIX ];
    fn2=[fn0 'GAssta.mat']
    save(fn2,'GAssta','URL')
end

if strcmp(R.SaveMode,'opendap')==1         
    for kt=1:length(GAssta.time)
clear D;
D.sst=squeeze(GAssta.GAssta(:,:,1,kt)); % zero altitude
D.latitude=GAssta.lat;
D.longitude=GAssta.lon;
D.time=GAssta.time(kt); % seconds since 1970
D.metadata.sst_variables={'latitude','longitude'};
D.metadata.url=URL;
D.metadata.attributes=A;
D.metadata.name='goes';
D.metadata.temporal='time_series'; %climatology
D.metadata.reference_time=datestr([1970,01,01,00,00,00],'yyyy-mm-dd HH:MM:SS');
readme=['time in seconds since reference_time; /' ...
        'missing values -9999999 replaced with NaN'];
D.metadata.readme = op_format_readme(readme);  %USE op_format_readme(readme) TO CONVERT.
D.metadata.user_friendly_time=GAssta.user_friendly_time(kt,:);
D.metadata.request_date = request.RequestDate;
D.metadata.request=request;

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
    KF=num2str(kt+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,D)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
    KF=num2str(kt+kf0,'%04d');
    FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
    FileName=FN3;
    save(FN3,'D')
end

    end %for kt=1:length(GAssta.time)
end % if strcmp(R.SaveMode,'opendap')==1

waitbar( 1.0 , hwaitbar, ['Acquiring GAssta'],'Name','GOES Download Progress');
close(hwaitbar)

disp('Data have been acquired.')
end % if nargin==1
%=====================================
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;
