function varargout = get_hycom(varargin)
% Gets Hybrid Coordinate Ocean Model (HYCOM) data.
%
%  Usage:
%
%  Get_HYCOM  
%
%  without arguments brings up GUI that allows a user to select variables, 
%  time, latitude, longitude, layer limits and save data to workspace or files. 
%  All selections are saved in a cell array Request.
%
%  Get_HYCOM(Request)
%
%  gets data selected by a prior call which was saved in the cell array Request.
%
%  Files: Get_HYCOM.m      - main function, this file
%         GUI_HYCOM.m      - GUI function
%         GUI_HYCOM.fig    - GUI graphic window
%         GUI_HYCOM_Request.mat    - contains the last Request
%  
% OPeNDAP Science Team
% Copyright 2007
% $Version 3.1.0$

%==========================================================================
% Meri Sheremet
% September 2007
%
% REVISION HISTORY:
% 2007/01/01 created, ms
% 2008/03/13 introduced DSI; cleared DSI and request from base workspace
% 2008/07/21 bipolar patch lat,lon 2d arrays; lon modulo 360 
%==========================================================================


if ~exist('loaddap')
errordlg('loaddap not found. Make sure it is in your MATLAB path.'); return
end               

if nargin<1
% no arguments passed, get parameters from GUI

h=msgbox('Acquiring metadata from HYCOM site, please wait ...');
% next line makes OK button invisible
hh=get(h,'Children'); set(hh(2),'Visible','off'); pause(1);
% Get HYCOM dataset dates from the list of files in the dens directory
dsi_hycom % disable temporary
close(h)

h = gui_hycom;
    if nargout>0
        varargout{1} = h; %handle is output.
    end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument Request, get HYCOM data  

request=varargin{1}; 
%assignin('base','request',request);
request.RequestDate=datestr(now,'yyyy-mm-dd HH:MM:SS');
R=request;

%example: R.DataSetBranch='OceanColor_MODIS_4km_8day_hdf';
% load dsi_OceanColor.mat DSI_OceanColor_MODIS_4km_8day_hdf
eval(['load dsi_hycom.mat DSI_' R.DataSetBranch ';'])
eval(['DSI=DSI_' R.DataSetBranch ';'])
eval(['clear DSI_' R.DataSetBranch ';'])

%assignin('base','DSI',DSI);

if length(R.DIRNAME)>0 & ~exist(R.DIRNAME);   
eval(['mkdir ' R.DIRNAME]);    
end
if sum(strcmp(R.Fields(:,end),'y'))==0 
errordlg('No variables selected.'); return
end
[L1,L2]=size(R.Fields);

% HYCOM variables are defined on a staggered grid.
% Corresponding lat, lon arrays may have different values.
% Make sure that the index ranges of all requested variables are same.
% Calculate index ranges based on dens, a variable defined at the center of a cell.
NT=length(DSI.Time);
NV=length(DSI.Fields(:,1));
Time=DSI.Time;
TimeA=DSI.TimeA;
TimeB=DSI.TimeB;
DATES=datestr(DSI.TimeA,'yyyymmdd');
Depth=DSI.Depth;

if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end
JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd')+1;
%assignin('base','Time',Time)
iDATE1=min(find(Time >= JD1));
iDATE2=max(find(Time <  JD2));

if(isempty(iDATE1) | isempty(iDATE2)) | (iDATE2<iDATE1)
    errordlg(['No data within the requested date range.']);
    return
end

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

if ~strcmp(DSI.DataSetBranch,'HYCOM_Atlantic_Ocean_Prediction_System') & R.LAT2 > 47
    LLATLON2D='y';
    
    if ~exist('Grid.mat','file')    
h=msgbox(['North of 47N HYCOM grid has a nonrectangular polar patch. Latitude and Longitude will be 2D arrays. ' ...
 'First time loading full Latitude and Longitude 2D arrays (64MB) from HYCOM website. Please wait.']);
% next line makes OK button invisible
hh=get(h,'Children'); set(hh(2),'Visible','off'); pause(3);

URL='http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis';
loaddap(['+v','-e'],[URL '?Latitude,Longitude'])
%dods_err = 0 means no error.
%dods_err = 1 means error.
%dods_err_msg [variable where error message is stored]
if dods_err
    disp(dods_err_msg)
    msgbox('Could not access dataset site. Sorry, try again later.');
    return
end
Latitude=Latitude.Latitude;
Longitude=Longitude.Longitude;
save Grid.mat Latitude Longitude
close(h)
    end

h=msgbox(['North of 47N HYCOM grid has a nonrectangular polar patch. Latitude and Longitude will be 2D arrays. ' ...
 'Loading full Latitude and Longitude 2D arrays from a local file. Please wait.']);
% next line makes OK button invisible
hh=get(h,'Children'); set(hh(2),'Visible','off'); pause(3);
load Grid.mat
%assignin('base','Latitude',Latitude);
%assignin('base','Longitude',Longitude);
close(h)

% fix values at discontinuities
Longitude(end,1:1125)=Longitude(1,1);
Longitude(end,1125+1:1125*3)=Longitude(1,1125*2+1);
Longitude(end,1125*3+1:1125*4)=Longitude(1,1125*4);

% adjust entered longitudes modulo 360 to match the dataset range 
LonM=0.5*(Longitude(1,1)+Longitude(1,end));
R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;

% find nonrectangular region that completely covers the selected rectangle
%disp('start searching')
%tic
[j11,i11]=gridval2ind2D(Latitude,Longitude,R.LAT1,R.LON1);
[j21,i21]=gridval2ind2D(Latitude,Longitude,R.LAT2,R.LON1);
[j12,i12]=gridval2ind2D(Latitude,Longitude,R.LAT1,R.LON2);
[j22,i22]=gridval2ind2D(Latitude,Longitude,R.LAT2,R.LON2);
%toc
%disp('searching complete')
j1=min([j11,j21,j12,j22]);
j2=max([j11,j21,j12,j22]);
i1=min([i11,i21,i12,i22]);
i2=max([i11,i21,i12,i22]);
% if spanning bipolar pole => latitude increases to maximum 
if ((i1<=1)&(i2>=1)) | ((i1<=1125*2)&(i2>=1125*2+1)) | ((i1<=1125*4)&(i2>=1125*4)) 
    j2=3298;
end
% if spanning bipolar equator => latitude decreases to minimum 
if ((i1<=1125)&(i2>=1125+1)) | ((i1<=1125*3)&(i2>=1125*3+1)) 
[j1125,i1125]=gridval2ind2D(Latitude,Longitude,R.LAT1,Longitude(1,1125));
j1=min([j11,j21,j12,j22,j1125]);
end


iLON1=i1-1;iLON2=i2-1;iLAT1=j1-1;iLAT2=j2-1;

if 1
% plot curvilinear grid
% and opendap rectangular area selection
incr=75;
figure(1); plot(Longitude(1:incr:end,1:incr:end),Latitude(1:incr:end,1:incr:end),'k.'); hold on
plot([R.LON1 R.LON2 R.LON2 R.LON1 R.LON1],[R.LAT1 R.LAT1 R.LAT2 R.LAT2 R.LAT1],'ro-')
plot(Longitude(j11,i11),Latitude(j11,i11),'b*')
plot(Longitude(j21,i21),Latitude(j21,i21),'b*')
plot(Longitude(j12,i12),Latitude(j12,i12),'b*')
plot(Longitude(j22,i22),Latitude(j22,i22),'b*')
plot(Longitude(j1:j2,i1),Latitude(j1:j2,i1),'b-')
plot(Longitude(j1:j2,i2),Latitude(j1:j2,i2),'b-')
plot(Longitude(j1,i1:i2),Latitude(j1,i1:i2),'b-')
plot(Longitude(j2,i1:i2),Latitude(j2,i1:i2),'b-')
axis([70 440 -90 90])
title('HYCOM grid with bipolar patch')
xlabel('Longitude')
ylabel('Latitude')
end

else % standard rectangular grid
Latitude=DSI.Latitude;
Longitude=DSI.Longitude;

LLATLON2D='n';
% adjust entered longitudes modulo 360 to match the dataset range 
LonM=0.5*(Longitude(1)+Longitude(end));
R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;
% find indices within the specified range of lons, lats
iLON1=min(find(Longitude >= R.LON1))-1;
iLON2=max(find(Longitude <= R.LON2))-1;
iLAT1=min(find(Latitude >= R.LAT1))-1;
iLAT2=max(find(Latitude <= R.LAT2))-1;

end

if(isempty(iLON1) | isempty(iLON2) | (iLON1>iLON2)) 
errordlg('Requested longitudes are out of range.'); return
end
if(isempty(iLAT1) | isempty(iLAT2) | (iLAT1>iLAT2)) 
errordlg('Requested latitudes are out of range.'); return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)|isempty(R.DATEINCR)|R.LATINCR==0|R.LONINCR==0|R.DATEINCR==0) 
    errordlg(['Subsampling values are invalid.']);
    return
end

Zstd=Depth;
% 33 standard levels 
% find range of indices for depth levels
% find if at least one 4D field is checked
L4D=0;
[N1,N2]=size(R.Fields);
for k=1:N1
    if R.Fields{k,N2}=='y' & DSI.Fields_Dim{k}==3
        L4D=1;
    end
end
if L4D==1 
    if (isempty(R.DEPTH1) | isempty(R.DEPTH2)) 
    errordlg(['Depth levels values are invalid or empty.']); return
    end
iLEVEL1=min(find(Depth >= R.DEPTH1))-1;
iLEVEL2=max(find(Depth <= R.DEPTH2))-1;
    if (isempty(iLEVEL1) | isempty(iLEVEL2) | isempty(R.LEVELINCR) ...
   |(iLEVEL1<0)|(iLEVEL2<0)|(iLEVEL1>32)|(iLEVEL2>32)|(iLEVEL1>iLEVEL2))    
errordlg('Requested depth levels are out of range.'); return
    end
else 
iLEVEL1=0;
iLEVEL2=0;
R.LEVELINCR=1;
end

%estimate the amount of requested data
[N1,N2]=size(R.Fields);
% select only checked variables
kV4=0;kV3=0;
for k=1:N1
    if R.Fields{k,end}=='y';
        if DSI.Fields_Dim{k}==2 % 2d fields
        kV3=kV3+1;
        elseif DSI.Fields_Dim{k}==3 % 3d fields
        kV4=kV4+1;
        end
    end
end
kDAT=(iDATE2-iDATE1)/R.DATEINCR+1;
kLAT=(iLAT2-iLAT1)/R.LATINCR+1;
kLON=(iLON2-iLON1)/R.LONINCR+1;
kLEV=(iLEVEL2-iLEVEL1)/R.LEVELINCR+1;
MB=round(kDAT*kLAT*kLON*(kV4*kLEV+kV3)*8/1024/1024);
MBLIMIT=100; % warning if requested amount of data > MBLIMIT
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Requested Data Amount','Continue','Cancel','Cancel');
    if strcmp(ButtonName,'Cancel') return; end
end

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end

% ***************************************************
% alternative saving mode for user convenience
% save time sequence of frames as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'opendap')==0
clear D3
L1=length((iDATE1:R.DATEINCR:iDATE2));L2=length((iLAT1:R.LATINCR:iLAT2));L3=length((iLON1:R.LONINCR:iLON2));
L4=length((iLEVEL1:R.LEVELINCR:iLEVEL2));
[N1,N2]=size(R.Fields);
for k=1:N1
    if R.Fields{k,end}=='y';
% select only checked variables
        if DSI.Fields_Dim{k}==2 % 2d fields
            FIELD=R.Fields{k,1};
eval(['D3.' FIELD '=zeros(L1,L2,L3)*NaN;']);
        elseif DSI.Fields_Dim{k}==3 % 3d fields
            FIELD=R.Fields{k,1};
eval(['D3.' FIELD '=zeros(L1,L2,L3,L4)*NaN;']);
        end
    end
end
D3.time=zeros(L1,1)*NaN;
end
% ********************************************************************

hwaitbar=waitbar(0,'Acquiring ','Name','HYCOM Download Progress');
kf=0;
URL0=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];
%A=loaddap('-A',URL0); 

for k=iDATE1:R.DATEINCR:iDATE2 % loop through dates
DATE=DATES(k,:);

kf=kf+1;    
clear D
% create standard blank opendap_nnnn data structure
for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
eval(['D.' FIELD '=NaN;']);
        end
end

% lat,lon,time are supposed to be the same for all fields
if LLATLON2D=='n'
D.latitude=Latitude(iLAT1+1:R.LATINCR:iLAT2+1);
D.longitude=Longitude(iLON1+1:R.LONINCR:iLON2+1);
else
D.latitude=Latitude(iLAT1+1:R.LATINCR:iLAT2+1,iLON1+1:R.LONINCR:iLON2+1);
D.longitude=Longitude(iLAT1+1:R.LATINCR:iLAT2+1,iLON1+1:R.LONINCR:iLON2+1);
end    
    if L4D==1
D.depth=Depth(iLEVEL1+1:R.LEVELINCR:iLEVEL2+1);
    end
%D.time=Time(k); % datenum;  
D.time=(Time(k)-datenum(1970,1,1,0,0,0))*86400;% in seconds since 1970-01-01 00:00:00

for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
%FIELDCOORD=cell2mat(DSI.Fields_Coord{kv})
%eval(['D.metadata.' FIELD '_variables={''latitude'',''longitude''};']);
%eval(['D.metadata.' FIELD '_variables=' 'DSI.FieldVariables.' FIELD ';']);
%eval(['D.metadata.' FIELD '_variables={' FIELDCOORD  '};']);
%eval(['D.metadata.fieldvariables.' FIELD '=DSI.FieldVariables.' FIELD ';']);
eval(['D.metadata.' FIELD '_variables=' 'DSI.Fields_Coord{kv};']);
        end
end
D.metadata.url='';
%D.metadata.attributes='';
%D.metadata.attributes=A;
D.metadata.attributes='';
D.metadata.name=lower(DSI.DataSetBranch);
D.metadata.temporal='time_series';
D.metadata.reference_time='1970-01-01 00:00:00';
D.metadata.readme=op_format_readme(DSI.Readme);
D.metadata.user_friendly_time=datestr(Time(k),'yyyy-mm-dd HH:MM:SS'); % Gregorian
%D.metadata.timeGregorian=NaN;
%D.metadata.time1970seconds=NaN;
D.metadata.request_date=R.RequestDate;
D.metadata.request=R;

kVV=0;
    for kV=1:L1 % loop through checked variables
        if R.Fields{kV,end}=='y'
            
            % a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCDEPTH URLCLAT URLCLON ])
%   constraints URLCTIME URLCDEPTH URLCLAT URLCLON  can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
%DSI.URLSITE='http://hycom.coaps.fsu.edu';
%DSI.URLPATH='/thredds/dodsC/';
%DSI.URLFILE='glb_analysis';

%A=loaddap('-A','http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis')

FIELD=R.Fields{kV,1};
URLCVAR=DSI.Fields_NameDS{kV};
FIELDR=[URLCVAR '.' URLCVAR];
VariableDim=DSI.Fields_DimDS{kV};
kVV=kVV+1;

xprogr=((kVV-1)/(kV4+kV3)+(k-iDATE1))/(iDATE2-iDATE1+1); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' strrep(FIELD,'_',' ') ' ' DATE],'Name','HYCOM Download Progress');


%C0='[0:1:0]'; % constraint time 
C0=['[' num2str(k-1) ':1:' num2str(k-1) ']']; % Constraint Time
C1=['[' num2str(iLEVEL1) ':' num2str(R.LEVELINCR) ':' num2str(iLEVEL2) ']']; % Constraint LEVEL
C2=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % Constraint Latitude
C3=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % Constraint Longitude

if VariableDim == 4
URL=[URL0 '?' URLCVAR C0 C1 C2 C3 ]; % 4D variables
else 
URL=[URL0 '?' URLCVAR C0 C2 C3 ];    % 3D variables
end
disp(URL)
loaddap(['+v','-e'],URL);
%dods_err = 0 means no error.
%dods_err = 1 means error.
%dods_err_msg [variable where error message is stored]
if dods_err
    disp(dods_err_msg)
    msgbox('Could not access dataset site. Sorry, try again later.');
    close(hwaitbar);
    return
end
A=loaddap(['+v','-A'],URL0);
D.metadata.attributes=A;

% replace missing values with NaN
%MV=eval(['A.' VariableName '.missing_value']);
%eval(['nn = find(' VariableName '.' VariableName '>1.0e+30);'])
%eval([VariableName '.' VariableName '(nn) = NaN;'])

% add URL for future reference
%eval([ VariableName '.url=URL;' ])
% add attributes as well 
%eval([ VariableName '.attributes=A;' ])

% current field is called f
eval(['f=squeeze(' FIELDR ');']);

% replace missing values with NaN according to formula stored in DSI.FormulaNaN
%eval(eval(['DSI.FormulaNaN.' FIELD]))
eval(eval(['DSI.FormulaNaN'])); % same for all fields
% convert to physical values according to formula stored in DSI.FormulaCnv
%eval(eval(['DSI.FormulaCnv.' FIELD]))

eval(['D.' FIELD '=f;']);
eval(['D.metadata.url.' FIELD '=URL;']);
%eval(['D.metadata.attributes.' FIELD '=A;']); % same for all fields moved
%outside the loop

if R.SaveWorkspace == 'y'& strcmp(R.SaveMode,'native')==1
%% save extracted data in workspace variables
eval(['tmp=' FIELD ';']);
% if you use a letter t instead of tmp,
% a mysterious window with a circle pops up 
%t
assignin('base',[FIELD '_' DATE],tmp)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'native')==1
% save extracted data to files
FN3=[R.FNPREFIX FIELD '_' DATE '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3
save(FN3,FIELD,'URL')
end

        end % if checked variables
    end % kV list of variables

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
KF=num2str(kf+kw0,'%04d');
WSN=['opendap_' KF];
assignin('base',WSN,D)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

if strcmp(R.SaveMode,'opendap')==0
    for kv=1:NV
        if R.Fields{kv,end}=='y'
            FIELD=R.Fields{kv,1};
            if DSI.Fields_Dim{kv}==2 % 2d fields
eval(['D3.' FIELD '(kf,:,:)=D.' FIELD ';']);
            elseif DSI.Fields_Dim{kv}==3 % 3d fields
eval(['D3.' FIELD '(kf,:,:,:)=D.' FIELD ';']);
            end
        end
    end
    D3.time(kf)=Time(k);
end

end % loop through dates

% *************************************************************
% alternative saving mode for user convenience
% save sst as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'opendap')==0
    D3.latitude=D.latitude;
    D3.longitude=D.longitude;
    if L4D==1
    D3.depth=D.depth;
    end

     %   D3.timeGregorian=Time(iTime,:);
    D3.metadata.coordinates={'time','latitude','longitude'};
    if L4D==1
    D3.metadata.coordinates={'time','latitude','longitude','depth'};
    end
    D3.metadata.name=DSI.DataSetBranch;
    D3.metadata.request=R;
end
if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==0
assignin('base','hycom',D3)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==0
% save extracted data to files
FN3=[R.FNPREFIX 'hycom' '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D3')
end

xprogr=1;
waitbar( xprogr , hwaitbar, ['Acquiring ' strrep(FIELD,'_',' ') ' ' DATE],'Name','HYCOM Download Progress');
close(hwaitbar)
%msgbox('The data have been acquired.')
disp('The data have been acquired.')
end % nargin

%=====================================
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;

%=======================================================
function [j,i]=gridval2ind2D(Latitude,Longitude,LAT,LON);
[NJ,NI]=size(Latitude);
% using a multigrid method for finding indices
%step 1
jincr=round(sqrt(NJ));
iincr=round(sqrt(NI));
jj=(1:jincr:NJ);
ii=(1:iincr:NI);
a=abs(Latitude(jj,ii)-LAT)+abs(Longitude(jj,ii)-LON); 
[ak k]=min(a(:)); 
[jk,ik]=ind2sub(size(a),k);
jjk=jj(jk);
iik=ii(ik);
% step 2
jj=max(1,jjk-jincr):1:min(NJ,jjk+jincr);
ii=max(1,iik-iincr):1:min(NI,iik+iincr);
a=abs(Latitude(jj,ii)-LAT)+abs(Longitude(jj,ii)-LON); 
[ak k]=min(a(:)); 
[jk,ik]=ind2sub(size(a),k);
jjk=jj(jk);
iik=ii(ik);
j=jjk;
i=iik;

