function varargout = get_oceancolor(varargin)
% Gets Ocean Color data.
% 
% get_oceancolor
% get_oceancolor(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007, 2008
% $Version 2.0.1$

%==========================================================================
% Meri Sheremet
% November 2007
%
% REVISION HISTORY:
% 2007/11/20 created
%==========================================================================

% check if loaddap is installed
if ~exist('loaddap','file')
    errordlg('loaddap not found. Make sure it is in your MATLAB path.')
    return
end

if nargin < 1 
h = gui_oceancolor;
    if nargout
        varargout{1} = h;
    end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
request=varargin{1}; 
request.RequestDate=datestr(now,'yyyy-mm-dd HH:MM:SS');
R=request;

%example: R.DataSetBranch='OceanColor_MODIS_4km_8day_hdf';
% load dsi_OceanColor.mat DSI_OceanColor_MODIS_4km_8day_hdf
eval(['load dsi_oceancolor.mat DSI_' R.DataSetBranch ';'])
eval(['DSI=DSI_' R.DataSetBranch ';'])
eval(['clear DSI_' R.DataSetBranch ';'])

%assignin('base','DSI',DSI);

NT=length(DSI.Time);
NV=length(DSI.Fields(:,1));
Time=DSI.Time;
TimeA=DSI.TimeA;
TimeB=DSI.TimeB;
Latitude=DSI.Latitude;
Longitude=DSI.Longitude;

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

% adjust entered longitudes modulo 360 to match the dataset range 
%LonM=0.5*(Longitude(1)+Longitude(end));
%R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
%R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;
% the above might not work if Longitude array is not monotonic

% Find indices within the specified range of lats and lons
% Note that array Latitude runs from North to South
iLAT2=max(find(Latitude >= R.LAT1))-1;
iLAT1=min(find(Latitude <= R.LAT2))-1;
% Note that Longitude may jump from 180E to -179W in Western Pacific Region
i=find(Longitude >= R.LON1);[dummy,j]=min(Longitude(i));iLON1=i(j);iLON1=iLON1-1;
i=find(Longitude <= R.LON2);[dummy,j]=max(Longitude(i));iLON2=i(j);iLON2=iLON2-1;

if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)|isempty(R.DATEINCR)|R.LATINCR==0|R.LONINCR==0|R.DATEINCR==0) 
    errordlg(['Subsampling values are invalid.']);
    return
end
if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end

JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd')+1;

%JDA=TimeA;
%JDB=TimeB;
%iDATE1=min(find(JDB > JD1));
%iDATE2=max(find(JDA < JD2));

JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd')+1;
%assignin('base','Time',Time)
iDATE1=min(find(Time >= JD1));
iDATE2=max(find(Time <  JD2));


%datestr(JD1,'yyyy-mm-dd HH:MM:SS');
%datestr(JD2,'yyyy-mm-dd HH:MM:SS');
%datestr(JDA(iDATE1),'yyyy-mm-dd HH:MM:SS');
%datestr(JDB(iDATE1),'yyyy-mm-dd HH:MM:SS');
%datestr(JDA(iDATE2),'yyyy-mm-dd HH:MM:SS');
%datestr(JDB(iDATE2),'yyyy-mm-dd HH:MM:SS');

if(isempty(iDATE1) | isempty(iDATE2)) | (iDATE2<iDATE1)
    errordlg(['No data within the requested date range.']);
    return
end
if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['No saving mode selected.']);
    return
end
if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end

% Estimate the amount of requested data
Nt=length(iDATE1:R.DATEINCR:iDATE2);
Ny=length(iLAT1:R.LATINCR:iLAT2);
Nx=length(iLON1:R.LONINCR:iLON2);
Nv=0;
for k=1:length(R.Fields(:,1))
    if R.Fields{k,end}=='y' Nv=Nv+1; end % number of checked fields
end
if Nv==0
    errordlg(['No variables have been selected.']);
    return
end
MB=round(Nt*Ny*Nx*Nv*8/1024/1024);
MBLIMIT=100;
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Requested Data Amount','Continue','Cancel','Cancel');
if strcmp(ButtonName,'Cancel') return; end
end

CLAT=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
CLON=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number: 
% kw0 in workspace; kf0 in files
[kw0,kf0]=LastFrameNum(R);
end

% ***************************************************
% alternative saving mode for user convenience
% save time sequence of frames as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'opendap')==0
clear D3
L1=length((iDATE1:R.DATEINCR:iDATE2));L2=length((iLAT1:R.LATINCR:iLAT2));L3=length((iLON1:R.LONINCR:iLON2));
% allocate memory for 3D array
    for kv=1:NV
        if R.Fields{kv,end}=='y'
            FIELD=R.Fields{kv,1};
eval(['D3.' FIELD '=zeros(L1,L2,L3)*NaN;']);
        end
    end
D3.time=zeros(L1,1)*NaN;
end
% ********************************************************************

% Progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','OceanColor Download Progress');
% browse through dates
kf=0;
for k=iDATE1:R.DATEINCR:iDATE2
kf=kf+1;    

clear D
% create standard blank opendap_nnnn data structure
for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
eval(['D.' FIELD '=NaN;']);
        end
end

% lat,lon,time are supposed to be the same for all fields
D.latitude=Latitude(iLAT1+1:R.LATINCR:iLAT2+1);
D.longitude=Longitude(iLON1+1:R.LONINCR:iLON2+1);
%D.time=Time(k); % datenum;  
D.time=(Time(k)-datenum(1970,1,1,0,0,0))*86400;% in seconds since 1970-01-01 00:00:00

for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
%eval(['D.metadata.' FIELD '_variables={''latitude'',''longitude''};']);
eval(['D.metadata.' FIELD '_variables=' 'DSI.FieldVariables.' FIELD ';']);
%eval(['D.metadata.fieldvariables.' FIELD '=DSI.FieldVariables.' FIELD ';']);
        end
end
D.metadata.url='';
D.metadata.attributes='';
D.metadata.name=lower(DSI.DataSetBranch);
D.metadata.temporal='time_series';
D.metadata.reference_time='1970-01-01 00:00:00';
D.metadata.readme=op_format_readme(DSI.Readme);
D.metadata.user_friendly_time=datestr(Time(k),'yyyy-mm-dd HH:MM:SS'); % Gregorian
%D.metadata.timeGregorian=NaN;
%D.metadata.time1970seconds=NaN;
D.metadata.request_date=R.RequestDate;
D.metadata.request=R;
          
    for kv=1:NV
        if R.Fields{kv,end}=='y'
% browse through checked variables    
FIELD=R.Fields{kv,1}; %
s=[datestr(Time(k),'yyyy-mm-dd') ' ' FIELD]; s=strrep(s,'_',' ');
xprogr=(k-iDATE1+(kv-1)/NV )/(iDATE2-iDATE1+1); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s ],'Name','OceanColor Download Progress');

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
%   constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get
        
URLFILE=eval(['DSI.URLFILE.' FIELD '{k};']);
CVAR=eval(['DSI.URLCVAR.' FIELD ';']);
            if ~isempty(URLFILE) & ~isempty(CVAR)
CTIME=DSI.URLCTIME;
URL=[DSI.URLSITE DSI.URLPATH URLFILE '?' CVAR CTIME CLAT CLON];
disp(URL)

loaddap(['+v','-e'],URL);
                if ~dods_err
clear A
A=loaddap('-A',URL);

% current field is called f
eval(['f=' DSI.Fields{kv,2} ';']);
% replace missing values with NaN according to formula stored in DSI.FormulaNaN
eval(eval(['DSI.FormulaNaN.' FIELD]))
% convert to physical values according to formula stored in DSI.FormulaCnv
eval(eval(['DSI.FormulaCnv.' FIELD]))

eval(['D.' FIELD '=f;']);
eval(['D.metadata.url.' FIELD '=URL;']);
eval(['D.metadata.attributes.' FIELD '=A;']);
                else
                disp('Warning: the file is corrupted')
                eval(['D.' FIELD '=NaN;']);
                eval(['D.metadata.url.' FIELD '='''';']);
                eval(['D.metadata.attributes.' FIELD '='''';']);
                end
            else % frame is missing 
            disp('Warning: the frame is missing')
            eval(['D.' FIELD '=NaN;']);
            eval(['D.metadata.url.' FIELD '='''';']);
            eval(['D.metadata.attributes.' FIELD '='''';']);
            end
        end
    end

%D.metadata.timeGregorian=datestr(Time(k),'yyyy-mm-dd HH:MM:SS');
%D.metadata.time1970seconds=(Time(k)-datenum(1970,1,1,0,0,0))*86400;
%D.metadata.timeGregorian=datestr(Time(k),'yyyy-mm-dd HH:MM:SS');
%D.metadata.time1970seconds=(Time(k)-datenum(1970,1,1,0,0,0))*86400;

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
    KF=num2str(kf+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,D)
end

if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

if strcmp(R.SaveMode,'opendap')==0
    for kv=1:NV
        if R.Fields{kv,end}=='y'
            FIELD=R.Fields{kv,1};
eval(['D3.' FIELD '(kf,:,:)=D.' FIELD ';']);
        end
    end
    D3.time(kf)=Time(k);
end

end %k 

% *************************************************************
% alternative saving mode for user convenience
% save sst as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'opendap')==0
    D3.latitude=D.latitude;
    D3.longitude=D.longitude;
 %   D3.timeGregorian=Time(iTime,:);
    D3.metadata.fieldvariables={'time','latitude','longitude'};
    D3.metadata.name=DSI.DataSetBranch;
    D3.metadata.request=R;
end
if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==0
assignin('base','oceancolor',D3)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==0
% save extracted data to files
FN3=[R.FNPREFIX 'oceancolor' '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D3')
end
xprogr=1; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring '],'Name','OceanColor Download Progress');
close(hwaitbar)
disp('Data have been acquired.')
end % nargin
% end get_oceancolor **************************************
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;

