function varargout = get_pathfinder4km(varargin)
% Gets Pathfinder 4-km sea surface temperature data, climatologies and time
% series.
%
% get_pathfinder4km
% get_pathfinder4km(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007
% $Version 2.1.4$

%==========================================================================
% Meri Sheremet
% REVISION HISTORY:
% 2007/01/01 created, ms
% 2007/10/23 opendap_nnnn structure
%==========================================================================

if nargin < 1

% no arguments passed, get parameters from GUI
h = gui_pathfinder4km;
if nargout
    varargout{1} = h; %handle is output.
end
elseif nargin > 1

    disp('Incorrect number of arguments.'); return  

else
% one argument request, get Pathfinder4km data

% Old websites
% URLBase='http://data.nodc.noaa.gov/cgi-bin/nph-dods/pathfinder/Version5.0';
% URLBase=['http://dods.jpl.nasa.gov/dods-bin/nph-hdf/pub/',...
%     'sea_surface_temperature/avhrr/pathfinder/data_v5'];

% Climatology is temporary at
%http://satdat1.gso.uri.edu/thredds/dodsC/NWAtlanticRaw_1km.html
URLBaseClim='http://satdat1.gso.uri.edu/opendap/sea_surface_temperature/climatology/pathfinder/Version5.0/';
URLBase='http://dods.jpl.nasa.gov/opendap/sea_surface_temperature/avhrr/pathfinder/data_v5';
% example of URL
%'http://dods.jpl.nasa.gov/opendap/sea_surface_temperature/avhrr/pathfinder
%/data_v5/daily/ascending/04km/2005/sst/2005001.s04d4pfrt-sst.hdf
%?sst[0:1:4095][0:1:8191]'
%URLLandMask='http://data.nodc.noaa.gov/pathfinder/Version5.0/pfv50_land.m04.hdf';
URLLandMask='http://satdat1.gso.uri.edu/opendap/Pathfinder/landmask/pfv50_land.m04.hdf';

MBLIMIT=100; % warning if requested amount of data exceeds that

request=varargin{1};
request.RequestDate=datestr(now,'yyyy-mm-dd HH:MM:SS');
R=request; 
%assignin('base','request',request);

tmp = mfilename;
tmp = strrep(lower(tmp),'get','gui');
tmp = strrep(tmp,'.m','');
load(strcat(tmp,'_filelist','.mat'),'Latitude','Longitude');

if ~exist('loaddap','file')
    errordlg('loaddap not found. Make sure it is in your MATLAB path.')
    return
end        
if ~isempty(R.Climatology)
    R.DATE1(1:4)='0000';R.DATE2(1:4)='0000';
    % if climatology request spans a new year, as in Dec-Jan period
    if str2num(R.DATE2)<str2num(R.DATE1) R.DATE2(1:4)='0001'; end 
end      
if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

% adjust entered longitudes modulo 360 to match the dataset range 
LonM=0.5*(Longitude(1)+Longitude(end));
R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;

% loaddap index convention 0:N-1
iLAT2=max(find(Latitude >= R.LAT1))-1;
iLAT1=min(find(Latitude <= R.LAT2))-1;
iLON1=min(find(Longitude >= R.LON1))-1;
iLON2=max(find(Longitude <= R.LON2))-1;
if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)) 
    errordlg(['Subsampling values are invalid.']);
    return
end
if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end
if R.Pass{1}=='n' & R.Pass{2}=='n' & R.Pass{3}=='n' & R.Pass{4}=='n'
    errordlg(['No satellite passes selected.']);
    return
end
if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['Select how to save data.']);
    return
end
if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end
if R.Clim_Fields{1,2}=='n' & R.Clim_Fields{2,2}=='n' & R.Clim_Fields{3,2}=='n' 
    R.Climatology=''; 
end
if R.Fields{1,2}=='n' & R.Fields{2,2}=='n' & R.Fields{3,2}=='n' & R.Fields{4,2}=='n' ...
 & R.Fields{5,2}=='n' & R.Fields{6,2}=='n' & R.Fields{7,2}=='n' & R.CalcAnomaly=='n' 
R.TimeSeries=''; 
end
if isempty(R.Climatology) & isempty(R.TimeSeries)
    errordlg(['No variables have been selected.'])
    return
end

C1=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % Constraint Latitude
C2=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % Constraint Longitude

JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd');

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Climatology
if ~isempty(R.Climatology)

% MATLAB 7.5 has a bug in datenum at year 0001-04-01
% use year 2000 as base instead to deal with climatology
% Note MATLAB datenum uses Julian calendar before October 4, 1582
% and Gregorian calendar after October 15, 1582
% 2000 (and 1600) is a leap year in both Julian and Gregorian calendars
% We need a leap year as a base year in order to access yearday 366
JD1=datenum(['20' R.DATE1(3:end)],'yyyymmdd');
JD2=datenum(['20' R.DATE2(3:end)],'yyyymmdd');
   
    
% both night and day passes    
PassDir='Combined';PassFN='combined';
if     R.Pass{2}=='n' & R.Pass{3}=='n' % no day passes
PassDir='Night';PassFN='night';
elseif R.Pass{1}=='n' & R.Pass{4}=='n' % no night passes
PassDir='Day';PassFN='day';
end

if strcmp(R.Climatology,'Daily')
p=zeros(366,1); 
  for k=JD1:R.DATEINCR:JD2
j=k-datenum(str2num(datestr(k,'yyyy')),1,1)+1; % yearday
p(j)=1;
  end
  for kF=1:1:length(p)
t='000';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['day' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'5day')
p=zeros(73,1);
  for k=JD1:1:JD2
j=k-datenum(str2num(datestr(k,'yyyy')),1,1)+1; % yearday
j=min(floor((j-1)/5)+1,73);
p(j)=1;
  end
  for kF=1:1:length(p)
t='00';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['pentad' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'7day')
p=zeros(52,1);
  for k=JD1:1:JD2
j=k-datenum(str2num(datestr(k,'yyyy')),1,1)+1; % yearday
j=min(floor((j-1)/7)+1,52);
p(j)=1;
  end
  for kF=1:1:length(p)
t='00';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['week' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'8day')
p=zeros(45,1);
  for k=JD1:1:JD2
j=k-datenum(str2num(datestr(k,'yyyy')),1,1)+1; % yearday
j=min(floor((j-1)/8)+1,45);
p(j)=1;
  end
  for kF=1:1:length(p)
t='00';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['octad' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'Monthly')
p=zeros(12,1);
  for k=JD1:1:JD2
j=str2num(datestr(k,'mm')); % month
p(j)=1;
  end
  for kF=1:1:length(p)
t='00';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['month' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'Seasonal')
p=zeros(4,1);
  for k=JD1:1:JD2
j=str2num(datestr(k,'mm')); % month
j=floor((j-1)/3)+1; % season
p(j)=1;
  end
  for kF=1:1:length(p)
t='0';t1=num2str(kF);t(end-length(t1)+1:end)=t1;    
fnk{kF}=['season' t '_' PassFN '.hdf'];    
  end
  
elseif strcmp(R.Climatology,'Annual')
p=zeros(1,1);
p(1)=1;
fnk{1}=['annual' '_' PassFN '.hdf'];    
end 

% Estimate the amount of requested data
kD=0;
for k=1:1:length(p) % browse through list of files
  if p(k)==1 kD=kD+1; end % number of dates
end  
kV=0;L=length(R.Clim_Fields);
for k=1:L(1)
    if R.Clim_Fields{k,2}=='y' kV=kV+1; end % number of variables
end
if R.ApplyLandMask=='y' kV=kV+1; end
MB=round((kD*kV+1)*(iLAT2-iLAT1)/R.LATINCR*(iLON2-iLON1)/R.LONINCR*16/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Pathfinder4km Requested Data Amount','Continue','Cancel','Cancel');
if strcmp(ButtonName,'Cancel') return; end
end

if R.ApplyLandMask=='y'   
disp([URLLandMask '?dsp_band_1' C1 C2])    
loaddap('+v',[URLLandMask '?dsp_band_1' C1 C2]);
if ~exist('dsp_band_1','var')
    msgbox('Could not access landmask.'); return
end
LandMask=dsp_band_1.dsp_band_1;
end

% progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','Pathfinder4km Download Progress');

% count files to be retrieved
nF=length(find(p==1));

kt=0; % counting fields for progress bar and save
kf=0; % counting fields for saving in opendap
for kF=1:1:length(p) % browse through list of files
  if p(kF)==1            
kt=kt+1;
xprogr=((kt-1)/nF); % progress fraction
% waitbar interprets _ in the file name as TeX underscore
%'Interpretation','Normal' does not seem to work
% fix by replacing '_' with ' '
s=fnk{kF};s=strrep(s,'_',' ');
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');

URL=[URLBaseClim '/' R.Climatology '/' PassDir '/' fnk{kF}];
disp(URL)
A=loaddap('-A',URL); % get attributes
if ~exist('A','var')
    msgbox('Could not access climatology file.'); return
end

for k=1:3 % compose a request of several fields
  if R.Clim_Fields{k,2}=='y'  
URL=[URL '?' R.Clim_Fields{k,1} C1 C2 ','];
dummy = R.Clim_Fields{k,1};
  end
end
URL=URL(1:end-1); % remove the last comma
disp(URL)
loaddap('+v',URL)
if ~exist(dummy,'var') %if one of the fields requested is present
    msgbox('Could not access climatology file.'); return
end

% make additional manipulations with fields
if R.Clim_Fields{1,2}=='y' % Clim_SST
Clim_SST.Clim_SST=Clim_SST.Clim_SST*A.Clim_SST.scale_factor+A.Clim_SST.add_offset; 
    if R.ApplyLandMask=='y'
    Clim_SST.Clim_SST_masked=Clim_SST.Clim_SST;
    Clim_SST.Clim_SST_masked(find(LandMask == 2))=NaN; % 0 water, 2 land
    end
end
if R.Clim_Fields{2,2}=='y' % Clim_StandardDeviation
Clim_StandardDeviation.Clim_StandardDeviation=...
Clim_StandardDeviation.Clim_StandardDeviation*A.Clim_StandardDeviation.scale_factor+A.Clim_StandardDeviation.add_offset; 
end

% create standard opendap_nnnn structure
clear D
if R.Clim_Fields{1,2}=='y' % Clim_SST
    D.Clim_SST=Clim_SST.Clim_SST;
    if R.ApplyLandMask=='y'
    D.Clim_SST_masked=Clim_SST.Clim_SST_masked;
    end
    latitude=Clim_SST.Latitude;
    longitude=Clim_SST.Longitude;
end
if R.Clim_Fields{2,2}=='y' % Clim_StandardDeviation
D.Clim_StandardDeviation=Clim_StandardDeviation.Clim_StandardDeviation;
latitude=Clim_StandardDeviation.Latitude;
longitude=Clim_StandardDeviation.Longitude;
end
if R.Clim_Fields{3,2}=='y' % Clim_Counts
D.Clim_Counts=Clim_Counts.Clim_Counts;
latitude=Clim_Counts.Latitude;
longitude=Clim_Counts.Longitude;
end

D.latitude=latitude;
D.longitude=longitude;
[tA tB tC]=tttavg(R.Climatology,kF);
D.time=tC*86400; % the middle of the averaging interval in seconds since the beginning of the year
TimePeriodCovered=A.Global_Attributes.HDF_GLOBAL.Time_20Period_20Covered;

%metadata
if R.Clim_Fields{1,2}=='y' % Clim_SST
    D.metadata.Clim_SST_variables={'latitude','longitude'};
    if R.ApplyLandMask=='y'
    D.metadata.Clim_SST_masked_variables={'latitude','longitude'};
    end
end
if R.Clim_Fields{2,2}=='y' % Clim_StandardDeviation
D.metadata.Clim_StandardDeviation_variables={'latitude','longitude'};
end
if R.Clim_Fields{3,2}=='y' % Clim_Counts
D.metadata.Clim_Counts_variables={'latitude','longitude'};
end

D.metadata.url=URL;
D.metadata.attributes=A;
D.metadata.name='pathfinder4km';
D.metadata.temporal='climatology';
D.metadata.reference_time=datestr(datenum([0 0 0 0 0 0]),'yyyy-mm-dd HH:MM:SS');
%S1=['Comment: bad data values replaced with ''NaN''.'];
%S2=['Scale: variables converted to physical units.'];
%S3=['Years Included in Climatology: ' A.Global_Attributes.HDF_GLOBAL.Years_20Included_20in_20Climatology];
%readme = [S1,'/',S2,'/',S3];
%D.metadata.readme = readme;
readme=['time in seconds since reference_time; /' ...
        'years included in climatology: 1985 - 2001; /'...        
        'sst converted to physical units; /' ...
        'bad values replaced with NaN'];
D.metadata.readme = op_format_readme(readme);
D.metadata.user_friendly_time=TimePeriodCovered;
D.metadata.request_date=request.RequestDate;
D.metadata.request=request;

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
    KF=num2str(kt+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,D)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kt+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

if R.SaveFiles=='y' & strcmp(R.SaveMode,'native')==1
fn2=[R.DIRNAME '/' R.FNPREFIX fnk{kF}(1:end-3) 'mat']
Vars='';
    for k=1:3
        if R.Clim_Fields{k,2}=='y'  
Vars=[Vars ' ' R.Clim_Fields{k,1} ' '];
        end
    end
eval( ['save ' fn2 Vars ' A URL'] )
end
if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'native')==1
    for k=1:3
        if R.Clim_Fields{k,2}=='y'
t=R.Clim_Fields{k,1};t1=fnk{kF}(1:end-4);
assignin('base',[t '_' t1],eval(t)) 
        end
    end
end

    end % if p(kF)==1 
end % for kF=1:1:length(p) % browse through list of files
 
close(hwaitbar) 

else % ~isempty(R.Climatology)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time Series
if R.ApplyQualMask ~= 'n' 
   R.Fields{7,2}='y'; %qual
end

%if strcmp(R.TimeSeries,'Daily') 
%load GUI_Pathfinder4km_FileList PFList; 
tmp = mfilename;
tmp = strrep(lower(tmp),'get','gui');
tmp = strrep(tmp,'.m','');
eval(['load ',strcat(tmp,'_filelist','.mat'),' PFList' R.TimeSeries ';']);
%PFList=PFListDaily;
eval(['PFList=PFList' R.TimeSeries ';']);

% sort filenames within a cell array
PFList=sort(PFList);
% convert from cell array to character array of fixed width
PFList=strvcat(PFList); 

% find position of the BITCODE in the first file name
iBITCODE=min(findstr(PFList(1,:),'.'))+1; % one chars right from the first dot

% find position of the DAYNIGHT in the first file name
iDAYNIGHT=min(findstr(PFList(1,:),'.'))+5; % five chars right from the first dot
PassList=PFList(:,iDAYNIGHT); nn=[];
% pick only filenames corresponding to the selected Passes
if R.Pass{1}=='y' nn=[nn;findstr(PassList','1')']; end
if R.Pass{2}=='y' nn=[nn;findstr(PassList','2')']; end
if R.Pass{3}=='y' nn=[nn;findstr(PassList','3')']; end
if R.Pass{4}=='y' nn=[nn;findstr(PassList','4')']; end
nn=sort(nn);
PFList=PFList(nn,:);

% calculate which files fall between the selected dates
if strcmp(R.TimeSeries,'Daily')
JDA=datenum(str2num(PFList(:,1:4)),1,0)+str2num(PFList(:,5:7));
JDB=JDA;
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
elseif strcmp(R.TimeSeries,'5day')
JDA=datenum(str2num(PFList(:,1:4)),1,0)+str2num(PFList(:,5:7));
JDB=datenum(str2num(PFList(:,9:12)),1,0)+str2num(PFList(:,13:15));
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
elseif strcmp(R.TimeSeries,'7day')
JDA=datenum(str2num(PFList(:,1:4)),1,0)+(str2num(PFList(:,5:6))-1)*7+1;
JDB=JDA+6;
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
elseif strcmp(R.TimeSeries,'8day')
JDA=datenum(str2num(PFList(:,1:4)),1,0)+str2num(PFList(:,5:7));
JDB=datenum(str2num(PFList(:,9:12)),1,0)+str2num(PFList(:,13:15));
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
elseif strcmp(R.TimeSeries,'Monthly')
JDA=datenum(str2num(PFList(:,1:4)),str2num(PFList(:,5:6)),1);
JDB=datenum(str2num(PFList(:,1:4)),str2num(PFList(:,5:6))+1,1)-1;
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
elseif strcmp(R.TimeSeries,'Yearly')
JDA=datenum(str2num(PFList(:,1:4)),1,1);
JDB=datenum(str2num(PFList(:,1:4))+1,1,1)-1;
kF1=min(find(JDB >= JD1));
kF2=max(find(JDA <= JD2));
end

if(isempty(kF1) || isempty(kF2)) 
    errordlg(['No data within selected date range.']);
    return
end

% Estimate the amount of requested data
kD=round((kF2-kF1+1)/R.DATEINCR); % number of requested dates/Passes
kV=0;L=size(R.Fields);
for k=1:L(1)
    if R.Fields{k,2}=='y' kV=kV+1; end % number of requested variables
end
NV=kV;
if R.ApplyLandMask=='y' || R.ApplyQualMask=='y' kV=kV+1; end
if R.CalcAnomaly=='y' kV=kV+1; end
MB=round((kD*kV+1)*(iLAT2-iLAT1)/R.LATINCR*(iLON2-iLON1)/R.LONINCR*16/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Pathfinder4km Requested Data Amount','Continue','Cancel','Cancel');
if strcmp(ButtonName,'Cancel') return; end
end

if R.ApplyLandMask=='y'
disp([URLLandMask '?dsp_band_1' C1 C2])    
loaddap('+v',[URLLandMask '?dsp_band_1' C1 C2]);
if ~exist('dsp_band_1','var')
    msgbox('Could not access landmask.'); return
end
LandMask=dsp_band_1.dsp_band_1;
end

if R.CalcAnomaly=='y'
% make sure that sst is requested whenever it is checked or not in GUI     
R.Fields{1,2}='y'; %sst
% get climatology reference field
PassDir='Combined';PassFN='combined';
if R.Pass{1}=='y' & R.Pass{3}=='n'
PassDir='Night';PassFN='night';
elseif R.Pass{3}=='y' & R.Pass{1}=='n'
PassDir='Day';PassFN='day';
end
fnc=['annual' '_' PassFN '.hdf'];    

URL=[URLBaseClim '/' 'Annual' '/' PassDir '/' fnc];
disp(URL)
A=loaddap('-A',URL); % get attributes

URL=[URL '?' 'Clim_SST' C1 C2];
disp(URL)
loaddap('+v',URL)
if ~exist('Clim_SST','var')
    msgbox('Could not access climatology field.'); return
end

Clim_SST.Clim_SST=Clim_SST.Clim_SST*A.Clim_SST.scale_factor+A.Clim_SST.add_offset; 
end

% Progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','Pathfinder4km Download Progress');
readme = '';
kf=0;kt=0;
for kF=kF1:R.DATEINCR:kF2
    kt=kt+1;
    
% set up standard opendap_nnnn structure for saving data
% initialize blank names in order to preserve order
clear D
%R.Fields={'sst','n';'bsst','n';'sdev','n';'mask1','n';'mask2','n';'num','n';'qual','n'};
if R.Fields{1,2}=='y'
FIELD=R.Fields{k,1}; % sst field is special: sst_masked, ssta, ssta_masked are derived from it.  
D.sst=NaN;
    if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
    D.sst_masked=NaN; 
    end
    if R.CalcAnomaly=='y' 
    D.ssta=NaN;
        if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
        D.ssta_masked=NaN;
        end
    end
end

[N1,N2]=size(R.Fields);
for k=2:N1 % cycle through other fields
FIELD=R.Fields{k,1};    
    if R.Fields{k,2}=='y' 
    eval(['D.' FIELD '=NaN;']); 
    end
end
    
D.latitude=NaN;
D.longitude=NaN;
D.time=NaN;
    
% metadata    
if R.Fields{1,2}=='y'
FIELD=R.Fields{k,1}; % sst field is special: sst_masked, ssta, ssta_masked are derived from it.  
D.metadata.sst_variables={'latitude','longitude'}; 
    if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
    D.metadata.sst_masked_variables={'latitude','longitude'}; 
    end
    if R.CalcAnomaly=='y' 
    D.metadata.ssta_variables={'latitude','longitude'}; 
        if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
        D.metadata.ssta_variables={'latitude','longitude'}; 
        end
    end
end
for k=2:N1
FIELD=R.Fields{k,1};    
    if R.Fields{k,2}=='y' 
    eval(['D.metadata.' FIELD '_variables={''latitude'',''longitude''};']); 
    end
end
   
D.metadata.url='';
D.metadata.attributes='';
D.metadata.name='pathfinder4km';
D.metadata.temporal='time_series';
D.metadata.reference_time=datestr([1970,01,01,00,00,00],'yyyy-mm-dd HH:MM:SS');
readme=['time in seconds since reference_time; /' ...
        'time is nominal, based on data file name; /' ...
        'day is middle of the averaging interval; /' ...
        'hour is local solar time of the pass; /' ...
        'sst converted to physical units; /' ...
        'bad values replaced with NaN'];
D.metadata.readme=op_format_readme(readme); 
D.metadata.user_friendly_time='';
D.metadata.request_date=request.RequestDate;
D.metadata.request=request;
% end setup standard opendap_nnnn structure        
        
fn1=deblank(PFList(kF,:)); % base file name of sst field 

kV=0;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
% waitbar interprets _ in the file name as TeX underscore
%'Interpretation','Normal' does not seem to work
% fix by replacing '_' with ' '
s=fn1;s(find(s=='_'))=' ';
s=fn1;s=strrep(s,'_',' ');
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');

DAYNIGHT=fn1(iDAYNIGHT);

if     strcmp(R.TimeSeries,'Daily')   fn00=[fn1(1:4) fn1(5:7) 'd' DAYNIGHT];
elseif strcmp(R.TimeSeries,'5day')    fn00=[fn1(1:4) fn1(5:7) 'p' DAYNIGHT];
elseif strcmp(R.TimeSeries,'7day')    fn00=[fn1(1:4) fn1(5:6) 'w' DAYNIGHT];
elseif strcmp(R.TimeSeries,'8day')    fn00=[fn1(1:4) fn1(5:7) 'o' DAYNIGHT];
elseif strcmp(R.TimeSeries,'Monthly') fn00=[fn1(1:4) fn1(5:6) 'm' DAYNIGHT];
elseif strcmp(R.TimeSeries,'Yearly')  fn00=[fn1(1:4)          'y' DAYNIGHT];
end

fn0=[R.DIRNAME '/' R.FNPREFIX fn00 '-'];
Year=fn1(1:4);
% find position of the dash before sst
idash=min(findstr(fn1(11:end),'-'))+10;
% find ADPass subdirectory 
ADPass='';
if     (DAYNIGHT=='1' | DAYNIGHT=='2') ADPass='descending'; 
elseif (DAYNIGHT=='3' | DAYNIGHT=='4') ADPass='ascending'; 
end

BITS=''; if strcmp(fn1(end-7:end),'-16b.hdf') BITS='-16b'; end

% aquire qual field first because it is used in applying quality mask 
if R.Fields{7,2}=='y' %qual
fn=[fn1(1:idash) 'qual.hdf']; fn(iBITCODE)='m';
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/qual/' fn '?qual' C1 C2];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
    if ~exist('qual','var'); %('idontknowthevariablename_here','var')
    msgbox('Could not access qual file.'); return
    end
A=loaddap('-A',URL);

D.qual=qual.qual;
D.latitude=qual.lat;
D.longitude=qual.lon;

% by mistake the time HHMMSS string is converted to number in the Attributes
% we need to convert it back to string
time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
    if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.qual=URL;
D.metadata.attributes.qual=A;
end

% bsst
if R.Fields{2,2}=='y' %bsst
fn=[fn1(1:idash) 'bsst' BITS '.hdf'];
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?bsst' C1 C2]
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/bsst/' fn '?bsst' C1 C2];

s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
if ~exist('bsst','var')
    msgbox('Could not access bsst file.'); return
end
A=loaddap('-A',URL);

bsst.bsst=bsst.bsst*A.bsst.scale_factor+A.bsst.add_off;

D.bsst=bsst.bsst;
D.latitude=bsst.lat;
D.longitude=bsst.lon;

%problem with bsst 7day
time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.bsst=URL;
D.metadata.attributes.bsst=A;

end % bsst

% sdev
% note that sdev field is called sst in data file by mistake
if R.Fields{3,2}=='y' %sdev
fn=[fn1(1:idash) 'sdev' BITS '.hdf'];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?sdev' C1 C2]
clear sst
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/sdev/' fn '?sst' C1 C2];
disp(URL)
loaddap('+v',URL)
% sdev field is named sst in Pathfinder hdf files: rename it
% trouble accessing some sdev files

    if exist('sst','var')    
A=loaddap('-A',URL);
sdev.sdev=sst.sst*A.sst.scale_factor+A.sst.add_off;

D.sdev=sst.sst;
D.latitude=sst.lat;
D.longitude=sst.lon;

time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.sdev=URL;
D.metadata.attributes.sdev=A;

    else
disp(['Warning: could not access sdev file: ' URL]);
D.sdev=NaN;
D.metadata.url.sdev=URL;
    end

end % sdev

% mask1
if R.Fields{4,2}=='y' %mask1
fn=[fn1(1:idash) 'msk1.hdf']; fn(iBITCODE)='m';
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?mask1' C1 C2]
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/msk1/' fn '?mask1' C1 C2];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
if ~exist('mask1','var')
    msgbox('Could not access mask1 file.'); return
end
A=loaddap('-A',URL);

D.mask1=mask1.mask1;
D.latitude=mask1.lat;
D.longitude=mask1.lon;

time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.mask1=URL;
D.metadata.attributes.mask1=A;

end % mask1

% mask2
if R.Fields{5,2}=='y' %mask2
fn=[fn1(1:idash) 'msk2.hdf']; fn(iBITCODE)='m';
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?mask2' C1 C2]
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/msk2/' fn '?mask2' C1 C2];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
if ~exist('mask2','var')
    msgbox('Could not access mask2 file.'); return
end
A=loaddap('-A',URL);

D.mask2=mask2.mask2;
D.latitude=mask2.lat;
D.longitude=mask2.lon;

time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.mask2=URL;
D.metadata.attributes.mask2=A;

end

% num
if R.Fields{6,2}=='y' %num
fn=[fn1(1:idash) 'num.hdf']; fn(iBITCODE)='m';
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?num' C1 C2]
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/nobs/' fn '?num' C1 C2];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
if ~exist('num','var')
    msgbox('Could not access num file.'); return
end
A=loaddap('-A',URL);

D.num=num.num;
D.latitude=num.lat;
D.longitude=num.lon;

time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.num=URL;
D.metadata.attributes.num=A;
end % num

% process sst last in order to overwrite latitude, longitude, time, URL,
% Attributes with those corresponding to sst
% sst
if R.Fields{1,2}=='y' %sst
fn=fn1;
%URL=[URLBase VERSION '/' R.TimeSeries '/' Year '/' fn '?sst' C1 C2]
URL=[URLBase '/' lower(R.TimeSeries) '/' ADPass '/04km/' Year '/sst/' fn '?sst' C1 C2];
s=fn;s(find(s=='_'))=' ';
s=fn;s=strrep(s,'_',' ');
kV=kV+1;
xprogr=((kF-kF1+(kV-1)/NV)/(kF2-kF1+1)); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');
disp(URL)
loaddap('+v',URL)
    if ~exist('sst','var')
    msgbox('Could not access sst file.'); return
    end
A=loaddap('-A',URL);
sst.sst=sst.sst*A.sst.scale_factor+A.sst.add_off;
D.sst=sst.sst;

    if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
    D.sst_masked=D.sst; 
    end
    if R.ApplyQualMask ~= 'n'
    t=str2num(R.ApplyQualMask);  
    D.sst_masked(find(D.qual <= t))=NaN;
    end
    if R.ApplyLandMask=='y'
    D.sst_masked(find(LandMask == 2))=NaN; % 0 water, 2 land
    end

D.latitude=sst.lat;
D.longitude=sst.lon;

%  A.Global_Attributes.HDF_GLOBAL.dsp_StartTime
%  A.Global_Attributes.HDF_GLOBAL.dsp_StartDate
time=num2str(round(A.Global_Attributes.HDF_GLOBAL.dsp_StartTime));
    if length(time)==5 time=['0' time]; end
time=[num2str(A.Global_Attributes.HDF_GLOBAL.dsp_StartDate) time];
% add URL and Attributes
D.metadata.url.sst=URL;
D.metadata.attributes.sst=A;
end % sst

if R.CalcAnomaly=='y' %ssta
D.ssta=D.sst-Clim_SST.Clim_SST;
    if (R.ApplyQualMask ~= 'n') | (R.ApplyLandMask=='y') 
    D.ssta_masked=D.ssta; 
    end
    if R.ApplyQualMask ~= 'n'
    t=str2num(R.ApplyQualMask);  
    D.ssta_masked(find(D.qual <= t))=NaN;
    end
    if R.ApplyLandMask=='y'
    D.ssta_masked(find(LandMask == 2))=NaN; % 0 water, 2 land
    end
end

% calculate nominal time based on the filename
LST=0; %local solar time
if     DAYNIGHT=='1' LST=2; 
elseif DAYNIGHT=='3' LST=14;  
elseif DAYNIGHT=='2' LST=10;  
elseif DAYNIGHT=='4' LST=22;  
end
if     strcmp(R.TimeSeries,'Daily')
    jyd=str2num(fn1(5:7)); % yearday
    time_nominal=datenum(fn1(1:4),'yyyy')+(jyd-1)+floor(1/2)+LST/24;    
elseif strcmp(R.TimeSeries,'5day')
    jyd=str2num(fn1(5:7)); % yearday
    time_nominal=datenum(fn1(1:4),'yyyy')+(jyd-1)+floor(5/2)+LST/24;     
elseif strcmp(R.TimeSeries,'7day')
    jw=str2num(fn1(5:6)); %week number
    jyd=(jw-1)*7+1; % yearday
    time_nominal=datenum(fn1(1:4),'yyyy')+(jyd-1)+floor(7/2)+LST/24;     
elseif strcmp(R.TimeSeries,'8day')
    jyd=str2num(fn1(5:7)); % yearday
    time_nominal=datenum(fn1(1:4),'yyyy')+(jyd-1)+floor(8/2)+LST/24;     
elseif strcmp(R.TimeSeries,'Monthly') 
    time_nominal=datenum([fn1(1:4) fn1(5:7)],'yyyymm')+floor(30/2)+LST/24;
elseif strcmp(R.TimeSeries,'Yearly')  
    time_nominal=datenum([fn1(1:4)],'yyyy')+floor(365/2)+LST/24;
end
%time_nominal=(JDA(kF)+JDB(kF)+1)/2;
time_nominal_gregorian=datestr(time_nominal,'yyyy-mm-dd HH:MM:SS');

% some hdf files have time missing
% in this case use time_nominal based on the file name
%if strcmp(time,'00')
%    time=datestr(time_nominal,'yyyymmddHHMMSS')
%end
% time in attributes is not right in many case; use time_nominal.
time=datestr(time_nominal,'yyyymmddHHMMSS');
D.time=86400*(datenum(time,'yyyymmddHHMMSS')-datenum('19700101000000','yyyymmddHHMMSS')); % seconds since 1970
D.metadata.user_friendly_time=datestr(datenum(time,'yyyymmddHHMMSS'),'yyyy-mm-dd HH:MM:SS'); %time;
%D.metadata.time_nominal_gregorian=time_nominal_gregorian;

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
    KF=num2str(kt+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,D)
end

if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kt+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'native')==1
assignin('base',['Pathfinder4km_' fn00],D) 
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'native')==1
fn2=[R.DIRNAME '/' R.FNPREFIX fn00 '.mat'];
save(fn2,'D')
end

end % kF
xprogr=1; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder4km Download Progress');

close(hwaitbar)

end % R.Climatology or R.TimeSeries

%msgbox('The data have been acquired.')
disp('Data have been acquired.')

end % narg
%==========================================================================
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number
%KF=num2str(kf,'%04d');
% latest workspace frame number
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
kw=k;
% the latest saved file 
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) kf=0;
else
a=a{end};
kf=str2num(a(length(R.FNPREFIX)+1:end-4));
end
%==========================================================================
function [tA tB tC]=tttavg(TAVG,k)
% for various temporal averages find the begining, end, and middle 
% of the averaging interval in days since Jan-01 00:00:00 
TAVG=lower(TAVG);
tA=NaN;tB=NaN;
if strcmp(TAVG,'daily') | strcmp(TAVG,'day')
    tA=k-1;tB=k;
elseif strcmp(TAVG,'5day') | strcmp(TAVG,'pentad')
    tA=(k-1)*5;tB=k*5;
elseif strcmp(TAVG,'7day') | strcmp(TAVG,'week')
    tA=(k-1)*7;tB=k*7;
elseif strcmp(TAVG,'8day') | strcmp(TAVG,'octad')
    tA=(k-1)*8;tB=k*8;
elseif strcmp(TAVG,'monthly') | strcmp(TAVG,'month')
    tA=datenum(0,k,0,0,0,0);tB=datenum(0,k+1,0,0,0,0);
elseif strcmp(TAVG,'seasonal') | strcmp(TAVG,'season')
    tA=datenum(0,(k-1)*3+1,0,0,0,0);tB=datenum(0,k*3+1,0,0,0,0);
elseif strcmp(TAVG,'annual') 
    tA=datenum(k-1,1,0,0,0,0);tB=datenum(k,1,0,0,0,0);
end
tC=0.5*(tA+tB);
%==========================================================================

